/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"
#include "HvControlVar.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_YY2znL7b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AnKShxRV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Cnpr0hiA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8pC5mtyS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_utclQRON_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_e3ts9VO9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_V6saHwuz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gtiw1FC9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Vox6ASxg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WSq03SI7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Pv4Zd52q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_yGAtQQ9s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_fN2yegJM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_iQj64Czz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Y48AnoYs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BV4tCIC2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_Ad4gbr2B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tfj9xwEq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TO6w9cKy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yVgL7z6Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2gFFtlVW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0nrOo9YP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_y3htgBAb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_kwZboviT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_6zAgo1z9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_sD5jrpnR_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_zMhZrShh;
  SignalLine sLine_atqufP6Z;
  SignalLine sLine_TrlxMKlQ;
  ControlBinop cBinop_YY2znL7b;
  ControlBinop cBinop_AnKShxRV;
  ControlBinop cBinop_Cnpr0hiA;
  ControlBinop cBinop_8pC5mtyS;
  ControlBinop cBinop_V6saHwuz;
  ControlBinop cBinop_Vox6ASxg;
  ControlBinop cBinop_WSq03SI7;
  ControlBinop cBinop_Pv4Zd52q;
  ControlVar cVar_yGAtQQ9s;
  ControlVar cVar_fN2yegJM;
  SignalVarf sVarf_qmsqoGux;
  SignalVarf sVarf_RbG2B2NR;
  SignalVarf sVarf_X2aTvGIr;
  SignalVarf sVarf_ryFvrcL7;
  ControlBinop cBinop_tfj9xwEq;
  ControlBinop cBinop_0nrOo9YP;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
